@@ -25,7 +25,7 @@ from libsonata import ElementReportReader, SonataError
 
 import bluepysnap._plotting
 from bluepysnap.exceptions import BluepySnapError
-from bluepysnap.utils import ensure_list
+from bluepysnap.utils import ensure_list, ensure_ids
 
 L = logging.getLogger(__name__)
 
@@ -45,14 +45,14 @@ def _get_reader(reader_report, cls):
     return cls(path)
 
 
-class PopulationFrameReport(object):
+class PopulationFrameReport:
     """Access to PopulationFrameReport data."""
 
     def __init__(self, frame_report, population_name):
         """Initializes a PopulationFrameReport object from a FrameReport.
 
         Args:
-            frame_report (FrameReport): FrameReport containing this spike report population.
+            frame_report (FrameReport): FrameReport containing this frame report population.
             population_name (str): the population name corresponding to this report.
 
         Returns:
@@ -90,7 +90,7 @@ class PopulationFrameReport(object):
         """Fetch data from the report.
 
         Args:
-            group (None/int/list/np.array/dict): Get spikes filtered by group. See NodePopulation.
+            group (None/int/list/np.array/dict): Get frames filtered by group. See NodePopulation.
             t_start (float): Include only frames occurring at or after this time.
             t_stop (float): Include only frames occurring at or before this time.
 
@@ -121,10 +121,10 @@ class PopulationFrameReport(object):
         Returns:
             np.Array: Numpy array containing the node_ids included in the report
         """
-        return np.sort(np.asarray(self._frame_population.get_node_ids(), dtype=np.int64))
+        return np.sort(ensure_ids(self._frame_population.get_node_ids()))
 
 
-class FilteredFrameReport(object):
+class FilteredFrameReport:
     """Access to filtered FrameReport data."""
 
     def __init__(self, frame_report, group=None, t_start=None, t_stop=None):
@@ -135,7 +135,7 @@ class FilteredFrameReport(object):
 
         Args:
             frame_report (FrameReport): The FrameReport to filter.
-            group (None/int/list/np.array/dict): Get spikes filtered by group. See NodePopulation.
+            group (None/int/list/np.array/dict): Get frames filtered by group. See NodePopulation.
             t_start (float): Include only frames occurring at or after this time.
             t_stop (float): Include only frames occurring at or before this time.
 
@@ -177,7 +177,7 @@ class FilteredFrameReport(object):
     trace = bluepysnap._plotting.frame_trace
 
 
-class FrameReport(object):
+class FrameReport:
     """Access to FrameReport data."""
 
     def __init__(self, simulation, report_name):
@@ -272,7 +272,7 @@ class FrameReport(object):
         from all the populations of a report.
 
         Args:
-            group (None/int/list/np.array/dict): Get spikes filtered by group. See NodePopulation.
+            group (None/int/list/np.array/dict): Get frames filtered by group. See NodePopulation.
             t_start (float): Include only frames occurring at or after this time.
             t_stop (float): Include only frames occurring at or before this time.
 
