@@ -20,118 +20,39 @@ import inspect
 from collections.abc import Mapping, Sequence
 from copy import deepcopy
 
+from more_itertools import first
 import libsonata
 import numpy as np
 import pandas as pd
 
 from cached_property import cached_property
 
+from bluepysnap import query
+from bluepysnap.network import NetworkObject
 from bluepysnap import utils
 from bluepysnap.exceptions import BluepySnapError
-from bluepysnap.sonata_constants import (DYNAMICS_PREFIX, NODE_ID_KEY,
-                                         POPULATION_KEY, Node, ConstContainer)
+from bluepysnap.sonata_constants import (DYNAMICS_PREFIX, Node, ConstContainer)
 from bluepysnap.circuit_ids import CircuitNodeId, CircuitNodeIds
-
-# this constant is not part of the sonata standard
-NODE_SET_KEY = "$node_set"
+from bluepysnap._doctools import AbstractDocSubstitutionMeta
 
 
-class Nodes(object):
+class Nodes(NetworkObject, metaclass=AbstractDocSubstitutionMeta,
+            source_word="NetworkObject", target_word="Node"):
     """The top level Nodes accessor."""
 
-    def __init__(self, circuit):
+    def __init__(self, circuit):  # pylint: disable=useless-super-delegation
         """Initialize the top level Nodes accessor."""
-        self._circuit = circuit
-        self._config = self._circuit.config['networks']['nodes']
-        self._populations = self._get_populations()
+        super().__init__(circuit)
 
-    def _get_populations(self):
-        """Collect the different NodePopulation."""
-        res = {}
-        for file_config in self._config:
-            storage = NodeStorage(file_config, self._circuit)
-            for population in storage.population_names:  # pylint: disable=not-an-iterable
-                if population in res:
-                    raise BluepySnapError("Duplicated node population: '%s'" % population)
-                res[population] = storage.population(population)
-        return res
-
-    @cached_property
-    def population_names(self):
-        """Returns all the population names from the Circuit."""
-        return sorted(self._populations)
-
-    @cached_property
-    def property_dtypes(self):
-        """Returns all the property dtypes for the Circuit."""
-        def _update(d, index, value):
-            if index not in d:
-                d[index] = value
-            elif d[index] != value:
-                raise BluepySnapError("Same property with different "
-                                      "dtype. {}: {}!= {}".format(index, value, d[index]))
-
-        res = dict()
-        for pop in self.populations():
-            for varname, dtype in pop.property_dtypes.iteritems():
-                _update(res, varname, dtype)
-        return pd.Series(res)
-
-    def keys(self):
-        """Returns iterator on the population names.
-
-        Made to simulate the behavior of a dict.keys().
-        """
-        for name in self.population_names:
-            yield name
-
-    def values(self):
-        """Returns iterator on the NodePopulations.
-
-        Made to simulate the behavior of a dict.values().
-        """
-        for name in self.population_names:
-            yield self[name]
-
-    def items(self):
-        """Returns iterator on the tuples (population name, NodePopulations).
-
-        Made to simulate the behavior of a dict.items().
-        """
-        for name in self.population_names:
-            yield name, self[name]
-
-    # helper renaming
-    names = keys
-    populations = values
-
-    def __getitem__(self, population_name):
-        """Access the NodePopulation corresponding to the population 'population_name'."""
-        try:
-            return self._populations[population_name]
-        except KeyError:
-            raise BluepySnapError("{} not a node population.".format(population_name))
-
-    def __iter__(self):
-        """Allows iteration over the different NodePopulation."""
-        return iter(self.names())
-
-    @cached_property
-    def size(self):
-        """Total number of nodes inside the circuit."""
-        return sum(pop.size for pop in self.populations())
-
-    @cached_property
-    def property_names(self):
-        """Returns all the properties present inside the circuit."""
-        return set(prop for pop in self.populations() for prop in pop.property_names)
+    def _collect_populations(self):
+        return self._get_populations(NodeStorage, self._config['networks']['nodes'])
 
     def property_values(self, prop):
-        """Returns all the values for a given property."""
-        return set(value for pop in self.populations() if prop in pop.property_names for value in
+        """Returns all the values for a given Nodes property."""
+        return set(value for pop in self.values() if prop in pop.property_names for value in
                    pop.property_values(prop))
 
-    def ids(self, group=None):
+    def ids(self, group=None, sample=None, limit=None):
         """Returns the CircuitNodeIds corresponding to the nodes from ``group``.
 
         Args:
@@ -151,6 +72,12 @@ class Nodes(object):
                 - ``mapping``: Returns a CircuitNodeIds object containing nodes matching a
                     properties filter.
                 - ``None``: return all node IDs of the circuit in a CircuitNodeIds object.
+            sample (int): If specified, randomly choose ``sample`` number of
+                IDs from the match result. If the size of the sample is greater than
+                the size of all the NodePopulations then all ids are taken and shuffled.
+            limit (int): If specified, return the first ``limit`` number of
+                IDs from the match result. If limit is greater than the size of all the populations,
+                all node IDs are returned.
 
         Returns:
             CircuitNodeIds: returns a CircuitNodeIds containing all the node IDs and the
@@ -164,6 +91,7 @@ class Nodes(object):
 
         Examples:
             The available group parameter values (example with 2 node populations pop1 and pop2):
+
             >>> nodes = circuit.nodes
             >>> nodes.ids(group=None)  #  returns all CircuitNodeIds from the circuit
             >>> node_ids = CircuitNodeIds.from_arrays(["pop1", "pop2"], [1, 3])
@@ -186,17 +114,10 @@ class Nodes(object):
             if diff.size != 0:
                 raise BluepySnapError("Population {} does not exist in the circuit.".format(diff))
 
-        ids = np.empty((0,), dtype=np.int64)
-        str_type = "<U{}".format(max(len(pop) for pop in self.population_names))
-        populations = np.empty((0,), dtype=str_type)
-        for name, pop in self.items():
-            pop_ids = pop.ids(group=group, raise_missing_property=False)
-            pops = np.full_like(pop_ids, fill_value=name, dtype=str_type)
-            ids = np.concatenate([ids, pop_ids])
-            populations = np.concatenate([populations, pops])
-        return CircuitNodeIds.from_arrays(populations, ids)
+        fun = lambda x: (x.ids(group, raise_missing_property=False), x.name)
+        return self._get_ids_from_pop(fun, CircuitNodeIds, sample=sample, limit=limit)
 
-    def get(self, group=None, properties=None):
+    def get(self, group=None, properties=None):   # pylint: disable=arguments-differ
         """Node properties as a pandas DataFrame.
 
         Args:
@@ -215,29 +136,12 @@ class Nodes(object):
             The NodePopulation.property_names function will give you all the usable properties
             for the `properties` argument.
         """
-        ids = self.ids(group)
         if properties is None:
             properties = self.property_names
-        properties = utils.ensure_list(properties)
-
-        unknown_props = set(properties) - self.property_names
-        if unknown_props:
-            raise BluepySnapError("Unknown properties required: {}".format(unknown_props))
-
-        res = pd.DataFrame(index=ids.index, columns=properties)
-        for name, pop in self.items():
-            global_pop_ids = ids.filter_population(name)
-            pop_ids = global_pop_ids.get_ids()
-            pop_properties = set(properties) & pop.property_names
-            # indices from NodePopulation and Node get functions are different so I cannot use
-            # a dataframe equal directly and properties have different types so cannot use a multi
-            # dim numpy array
-            for prop in pop_properties:
-                res.loc[global_pop_ids.index, prop] = pop.get(pop_ids, properties=prop).to_numpy()
-        return res.sort_index()
+        return super().get(group, properties)
 
 
-class NodeStorage(object):
+class NodeStorage:
     """Node storage access."""
 
     def __init__(self, config, circuit):
@@ -310,19 +214,7 @@ class NodeStorage(object):
         return result
 
 
-# TODO: move to `libsonata` library
-def _complex_query(prop, query):
-    # pylint: disable=assignment-from-no-return
-    result = np.full(len(prop), True)
-    for key, value in query.items():
-        if key == '$regex':
-            result = np.logical_and(result, prop.str.match(value + "\\Z"))
-        else:
-            raise BluepySnapError("Unknown query modifier: '%s'" % key)
-    return result
-
-
-class NodePopulation(object):
+class NodePopulation:
     """Node population access."""
 
     def __init__(self, node_storage, population_name):
@@ -478,89 +370,17 @@ class NodePopulation(object):
             raise BluepySnapError("Undefined node set: '%s'" % node_set_name)
         return self._node_sets[node_set_name]
 
-    def _positional_mask(self, node_ids):
-        """Positional mask for the node IDs.
+    def _resolve_nodesets(self, queries):
+        def _resolve(queries, queries_key):
+            if queries_key == query.NODE_SET_KEY:
+                if query.AND_KEY not in queries:
+                    queries[query.AND_KEY] = []
+                queries[query.AND_KEY].append(self._get_node_set(queries[queries_key]))
+                del queries[queries_key]
 
-        Args:
-            node_ids (None/numpy.ndarray): the ids array. If None all ids are selected.
-
-        Examples:
-            if the data set contains 5 nodes:
-            _positional_mask([0,2]) --> [True, False, True, False, False]
-        """
-        if node_ids is None:
-            return np.full(len(self._data), fill_value=True)
-        mask = np.full(len(self._data), fill_value=False)
-        valid_node_ids = pd.Index(utils.ensure_list(node_ids)).intersection(self._data.index)
-        mask[valid_node_ids] = True
-        return mask
-
-    def _circuit_mask(self, queries):
-        """Handle the population, node ID and node set queries."""
-        populations = queries.pop(POPULATION_KEY, None)
-        if populations is not None and self.name not in set(utils.ensure_list(populations)):
-            node_ids = []
-        else:
-            node_ids = queries.pop(NODE_ID_KEY, None)
-        node_set = queries.pop(NODE_SET_KEY, None)
-        if node_set is not None:
-            if not isinstance(node_set, str):
-                raise BluepySnapError("{} is not a valid node set name.".format(node_set))
-            node_ids = node_ids if node_ids else self._data.index.values
-            node_ids = np.intersect1d(node_ids, self.ids(node_set))
-        return queries, self._positional_mask(node_ids)
-
-    def _properties_mask(self, queries, raise_missing_prop):
-        """Return mask of node IDs with rows matching `props` dict."""
-        # pylint: disable=assignment-from-no-return
-        circuit_keys = {POPULATION_KEY, NODE_ID_KEY, NODE_SET_KEY}
-        unknown_props = set(queries) - set(self._data.columns) - circuit_keys
-        if unknown_props:
-            if raise_missing_prop:
-                raise BluepySnapError(
-                    "Unknown node properties: [{0}]".format(", ".join(unknown_props)))
-            return np.full(len(self._data), fill_value=False)
-
-        queries, mask = self._circuit_mask(queries)
-        if not mask.any():
-            # Avoid fail and/or processing time if wrong population or no nodes
-            return mask
-
-        for prop, values in queries.items():
-            prop = self._data[prop]
-            if np.issubdtype(prop.dtype.type, np.floating):
-                v1, v2 = values
-                prop_mask = np.logical_and(prop >= v1, prop <= v2)
-            elif isinstance(values, str) and values.startswith('regex:'):
-                prop_mask = _complex_query(prop, {'$regex': values[6:]})
-            elif isinstance(values, Mapping):
-                prop_mask = _complex_query(prop, values)
-            else:
-                prop_mask = np.in1d(prop, values)
-            mask = np.logical_and(mask, prop_mask)
-        return mask
-
-    def _operator_mask(self, queries, raise_missing_prop):
-        """Handle the query operators '$or', '$and'."""
-        if len(queries) == 0:
-            return np.full(len(self._data), True)
-
-        # will pop the population and or/and operators so need to copy
-        queries = deepcopy(queries)
-        first_key = list(queries)[0]
-        if first_key == '$or':
-            queries = queries.pop("$or")
-            operator = np.logical_or
-        elif first_key == '$and':
-            queries = queries.pop("$and")
-            operator = np.logical_and
-        else:
-            return self._properties_mask(queries, raise_missing_prop)
-
-        mask = np.full(len(self._data), first_key != "$or")
-        for query in queries:
-            mask = operator(mask, self._operator_mask(query, raise_missing_prop))
-        return mask
+        resolved_queries = deepcopy(queries)
+        query.traverse_queries_bottom_up(resolved_queries, _resolve)
+        return resolved_queries
 
     def _node_ids_by_filter(self, queries, raise_missing_prop):
         """Return node IDs if their properties match the `queries` dict.
@@ -579,7 +399,14 @@ class NodePopulation(object):
             >>>                              { Node.X: (0, 1), Node.MTYPE: 'L1_SLAC' }]})
 
         """
-        return self._data.index[self._operator_mask(queries, raise_missing_prop)].values
+        queries = self._resolve_nodesets(queries)
+        if raise_missing_prop:
+            properties = query.get_properties(queries)
+            if not properties.issubset(self._data.columns):
+                unknown_props = properties - set(self._data.columns)
+                raise BluepySnapError(f"Unknown node properties: {unknown_props}")
+        idx = query.resolve_ids(self._data, self.name, queries)
+        return self._data.index[idx].values
 
     def ids(self, group=None, limit=None, sample=None, raise_missing_property=True):
         """Node IDs corresponding to node ``group``.
@@ -589,7 +416,7 @@ class NodePopulation(object):
                 returned depends on the type of the ``group`` argument:
 
                 - ``int``, ``CircuitNodeId``: return a single node ID if it belongs to the circuit.
-                - ``CircuitNodeIds`` return IDs of nodes in an array.
+                - ``CircuitNodeIds`` return IDs of nodes from the node population in an array.
                 - ``sequence``: return IDs of nodes in an array.
                 - ``str``: return IDs of nodes in a node set.
                 - ``mapping``: return IDs of nodes matching a properties filter.
@@ -653,7 +480,8 @@ class NodePopulation(object):
             preserve_order = True
         else:
             result = utils.ensure_list(group)
-            if isinstance(next(iter(result), None), CircuitNodeId):
+            # test if first value is a CircuitNodeId all values are all CircuitNodeId
+            if isinstance(first(result, None), CircuitNodeId):
                 try:
                     result = [cid.id for cid in result if cid.population == self.name]
                 except AttributeError:
@@ -669,7 +497,7 @@ class NodePopulation(object):
         if limit is not None:
             result = result[:limit]
 
-        result = np.array(result, dtype=np.int64)
+        result = utils.ensure_ids(result)
         if preserve_order:
             return result
         else:
@@ -826,3 +654,9 @@ class NodePopulation(object):
             self._node_storage.circuit.config['components']['morphologies_dir'],
             self
         )
+
+    @cached_property
+    def models(self):
+        """Access to node neuron models."""
+        from bluepysnap.neuron_models import NeuronModelsHelper
+        return NeuronModelsHelper(self._node_storage.circuit.config['components'], self)
