@@ -19,13 +19,33 @@
 
 from collections.abc import Iterable
 import json
+import warnings
 
 import numpy as np
 
-from bluepysnap.exceptions import BluepySnapError
-from bluepysnap.circuit_ids import CircuitNodeId
+from bluepysnap.exceptions import (BluepySnapError, BluepySnapDeprecationWarning,
+                                   BluepySnapDeprecationError)
+from bluepysnap.circuit_ids import CircuitNodeId, CircuitEdgeId
 from bluepysnap.sonata_constants import DYNAMICS_PREFIX
 
+# dtypes for the different node and edge ids. We are using np.int64 to avoid the infamous
+# https://github.com/numpy/numpy/issues/15084 numpy problem. This type needs to be used for
+# all returned node or edge ids.
+IDS_DTYPE = np.int64
+
+
+class Deprecate:
+    """Class for the deprecations in BluepySnap."""
+    @staticmethod
+    def fail(msg=""):
+        """Raise a deprecation exception."""
+        raise BluepySnapDeprecationError(msg)
+
+    @staticmethod
+    def warn(msg=""):
+        """Issue a deprecation warning."""
+        warnings.warn(msg, BluepySnapDeprecationWarning)
+
 
 def load_json(filepath):
     """Load JSON from file."""
@@ -34,8 +54,8 @@ def load_json(filepath):
 
 
 def is_iterable(v):
-    """Check if `v` is any iterable (strings are considered scalar and CircuitNodeId also)."""
-    return isinstance(v, Iterable) and not isinstance(v, (str, CircuitNodeId))
+    """Check if `v` is any iterable (strings are considered scalar and CircuitNode/EdgeId also)."""
+    return isinstance(v, Iterable) and not isinstance(v, (str, CircuitNodeId, CircuitEdgeId))
 
 
 def ensure_list(v):
@@ -46,6 +66,19 @@ def ensure_list(v):
         return [v]
 
 
+def ensure_ids(a):
+    """Convert a numpy array dtype into IDS_DTYPE.
+
+    This function is here due to the https://github.com/numpy/numpy/issues/15084 numpy issue.
+    It is quite unsafe to the use uint64 for the ids due to this problem where :
+    numpy.uint64 + int --> float64
+    numpy.uint64 += int --> float64
+
+    This function needs to be used everywhere node_ids or edge_ids are returned.
+    """
+    return np.asarray(a, IDS_DTYPE)
+
+
 def add_dynamic_prefix(properties):
     """Add the dynamic prefix to a list of properties."""
     return [DYNAMICS_PREFIX + name for name in list(properties)]
